@@ -1,9 +1,18 @@
 #ifndef UVUTIL_HPP
 #define UVUTIL_HPP
 
+#include "thread.h"
 #include <string>
 #include <vector>
-#include <uv.h>
+#include "libuv/include/uv.h"
+
+/* Prevent naming conflicts for Free() and Calloc() */
+#define R_NO_REMAP
+// This could be defined from Makevars.win. In that case, don't re-define it.
+#ifndef STRICT_R_HEADERS
+#define STRICT_R_HEADERS
+#endif
+#include <Rcpp.h>
 
 inline uv_handle_t* toHandle(uv_timer_t* timer) {
   return (uv_handle_t*)timer;
@@ -19,10 +28,6 @@ inline uv_stream_t* toStream(uv_tcp_t* tcp) {
   return (uv_stream_t*)tcp;
 }
 
-void throwLastError(uv_loop_t* pLoop,
-  const std::string& prefix = std::string(),
-  const std::string& suffix = std::string());
-
 class WriteOp;
 
 // Abstract class for synchronously streaming known-length data
@@ -43,6 +48,14 @@ private:
 public:
   explicit InMemoryDataSource(const std::vector<uint8_t>& buffer = std::vector<uint8_t>())
     : _buffer(buffer), _pos(0) {}
+
+  explicit InMemoryDataSource(const Rcpp::RawVector& rawVector)
+    : _buffer(rawVector.size()), _pos(0) 
+  {
+    ASSERT_MAIN_THREAD()
+    std::copy(rawVector.begin(), rawVector.end(), _buffer.begin());
+  }
+
   virtual ~InMemoryDataSource() {}
   uint64_t size() const;
   uv_buf_t getData(size_t bytesDesired);
@@ -77,4 +90,18 @@ protected:
 
 };
 
+
+inline int ipFamily(const std::string& ip) {
+  // A buffer big enough for an IPv6 address
+  unsigned char addr[16];
+
+  if (uv_inet_pton(AF_INET6, ip.c_str(), addr) == 0) {
+    return AF_INET6;
+  }
+  if (uv_inet_pton(AF_INET, ip.c_str(), addr) == 0) {
+    return AF_INET;
+  }
+  return -1;
+}
+
 #endif // UVUTIL_HPP
